from typing import Protocol


def sms_decorator(func, to):
    from ..models import SMSMessage

    def wrapper():
        result = func()
        if result:
            SMSMessage.objects.create(phone_number=to)

    return wrapper


class SMSProviderClass(Protocol):
    to: str
    message: str
    conf: dict

    def send_sms(self) -> None:
        pass


class SMSProvider:
    def __getattribute__(self, item):
        element = super().__getattribute__(item)
        if callable(element) and item == "send_sms":
            return sms_decorator(element, self.to)

        return element

    def __init__(self, to, message, conf):
        self.to = to
        self.message = message
        self.conf = conf

    def send_sms(self) -> str:
        raise NotImplementedError()



from rest_framework import status
from rest_framework.response import Response

from ..conf import conf


class Error:
    def __init__(self, message):
        self.message = message

    def render(self):
        error = {"code": 1000, "message": self.message, "errors": []}

        return error


class ResponsesMixin:
    def simple_text_response(self, message=None):
        if message is None:
            message = conf.SMS_REQUEST_SUCCESS
        data = {"detail": message}

        return Response(data, status=status.HTTP_200_OK)

    def success_objects_response(self, data):
        return Response(data, status=status.HTTP_200_OK)

    def error_response(self, error_message):
        error = error_message
        if type(error_message) is str:
            error = Error(error_message).render()

        return Response(error, status=status.HTTP_400_BAD_REQUEST)



from django.contrib.admin import ModelAdmin, register

from .models import *


@register(SMSMessage)
class SMSMessageAdmin(ModelAdmin):
    readonly_fields = (
        "created",
        "phone_number",
    )

    def has_add_permission(self, request):
        return False


@register(PhoneCode)
class PhoneCodeAdmin(ModelAdmin):
    readonly_fields = (
        "valid_to",
        "created_at",
    )


# Generated by Django 3.0 on 2021-02-24 14:38

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import phonenumber_field.modelfields
import sms_auth.utils


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='SMSMessage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('phone_number', models.CharField(max_length=20, verbose_name='Phone number')),
            ],
            options={
                'verbose_name': 'Sms log',
                'verbose_name_plural': 'Sms log',
            },
        ),
        migrations.CreateModel(
            name='PhoneCode',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('phone_number', phonenumber_field.modelfields.PhoneNumberField(max_length=128, region=None, unique=True)),
                ('code', models.PositiveIntegerField(default=sms_auth.utils.random_code)),
                ('valid_to', models.DateTimeField(default=sms_auth.utils.valid_to)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('owner', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Phone code',
                'verbose_name_plural': 'Phone codes',
                'ordering': ('created_at',),
            },
        ),
    ]
